@@ -21,21 +21,27 @@ Example:
     print(result)
 """
 
+from importlib.metadata import version as get_version
+
+from .api import fetch_environments
 from .automation import ScaleWoBAutomation
 from .exceptions import (
     BrowserError,
     CommandError,
     EvaluationError,
+    NetworkError,
     ScaleWoBError,
     TimeoutError,
 )
 
-__version__ = "0.1.0"
+__version__ = get_version(__package__)  # type: ignore
 __all__ = [
     "ScaleWoBAutomation",
+    "fetch_environments",
     "ScaleWoBError",
     "TimeoutError",
     "CommandError",
     "EvaluationError",
     "BrowserError",
+    "NetworkError",
 ]
